/*
Copyright(C) 2007 Giada Giorgi

This file is part of X-Simulator.

    X-Simulator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    X-Simulator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <omnetpp.h>
#include "Packet_m.h"
#include "Event_m.h"
#include "Constant.h"

/*
-------------------------------------------------------------------------------
BUFFER
-------------------------------------------------------------------------------
*/
class Buffer3:public cSimpleModule{
protected:
	virtual void initialize();
	virtual void handleMessage(cMessage *msg);
	virtual void finish();
	virtual void updateDisplay();
private:
	cQueue queue;
	cMessage *tmp;
	int pacchetti;
	int byte;
	double rate;
	double T;
	double Tlat;
	double Ts;
	cOutVector pckVec;
	cOutVector byteVec;
};

Define_Module(Buffer3);

void Buffer3::initialize(){
	// ---------------------------------------------------------------------------
	// Ts rappresenta il tempo che impiega il buffer per trasmettere il pacchetto,
	//    questo tempo  viene calcolato come rapporto tra la lunghessa in bit del
	//    pacchetto e bit rate dello switch.
	// ---------------------------------------------------------------------------
	ev << "BUFFER : Inizializzazione ...\n";
	queue.setName("buffer");
	pckVec.setName("pacchetti");
	byteVec.setName("byte");
	pacchetti = 0;			 // numero di pacchetti in coda
	byte = 0;				 // numero di byte occupati
	Ts = 0;				 // tempo impiegato per spedire un pacchetto
	rate=par("rate");	 // [bit/s]
	if(rate==0)
		T=0;
	else
		T=1/rate;
	Tlat=par("latenza"); // [s] latenza: impostare a zero
	if(ev.isGUI()){updateDisplay();}
	ev << "BUFFER : Fine inizializzazione.\n";	
}

void Buffer3::handleMessage(cMessage *msg){
	if(msg->isSelfMessage()){
		tmp = (cMessage *) queue.pop();
		pacchetti = pacchetti -1;
		byte = byte - tmp->byteLength();
		send(tmp,"out");
		ev << "BUFFER : Ora di invio del pacchetto = " << simTime()*1000 << " [ms]\n";
		if(!queue.empty()){
			Ts = ((cMessage *)queue.head())->byteLength()*8*T+Tlat;
			scheduleAt(simTime()+Ts, new cMessage("BufTimer"));
		}
		delete msg;
	}
	else{
		ev << "BUFFER : Ora di arrivo del pacchetto = " << simTime()*1000 << " [ms]\n";
		if(!queue.empty()){
			//Inserisco il messaggio arrivato nel buffer.
			queue.insert(msg);
			pacchetti = pacchetti +1;
			byte = byte + msg->byteLength();
		}else{
			//Calcolo il tempo del servizio del messaggio arrivato.
			Ts = msg->byteLength()*8*T+Tlat;
			ev << "BUFFER : Tempo di servizio Ts = " << Ts << "\n";
			//Inserisco il messaggio arrivato nel buffer.
			queue.insert(msg);
			pacchetti = pacchetti +1;
			byte = byte + msg->byteLength();
			//Imposto il timer per spedire il messaggio.
			scheduleAt(simTime()+Ts, new cMessage("BufTimer"));
		}
	}
	if(ev.isGUI()){updateDisplay();}
	/*********************************
	 **    Salvataggio valori.      **
	 *********************************/
	pckVec.record(pacchetti);
	byteVec.record(byte);
}

void Buffer3::finish(){
	ev << "BUFFER : finish...\n";
	ev << "BUFFER : Rimozione di tutti i messaggi in coda...\n";
	while(!queue.empty()){
		delete (cMessage *)queue.pop();
	}
	ev << "BUFFER : Tutti i messaggi sono stati rimossi.\n";
}

void Buffer3::updateDisplay(){
	char buf[40];
	sprintf(buf, "pck: %d   byte: %d\n",pacchetti,byte);
	displayString().setTagArg("t",0,buf);
}