/*
Copyright(C) 2007 Giada Giorgi

This file is part of X-Simulator.

    X-Simulator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    X-Simulator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <iostream>
#include <fstream>
#include <omnetpp.h>
#include <math.h>
#include "Packet_m.h"
#include "Constant.h"
using namespace std;

class Chaoticsource:public cSimpleModule{
protected:
	virtual void initialize();
	virtual void handleMessage(cMessage *msg);
	virtual void finish();
	virtual void openfile();
	virtual void closefile();
private:
	double T;
	double d;
	double x;
	double y;
	double m1;
	double m2;
	double wlen;
	int sink;
	int address;
	ofstream youtFile;
	ofstream xoutFile;
};

Define_Module(Chaoticsource);

void Chaoticsource::initialize(){

	/*Inizializzazione dei parametri. */
	T  = par("T");
	d  = par("d");
	m1 = par("m1");
	m2 = par("m2") ;
	wlen = par("wlen");

	/*Inizializzazione variabili.*/
	x = 0.5;
	y = 0;
	openfile();
	if(!youtFile){ev << "SOURCE: Errore apertura file" << endl;}
	else{youtFile << y << endl;	}
	if(!xoutFile){ev << "SOURCE: Errore apertura file" << endl;}
	else{xoutFile << x << endl; }
	address = id();

	/*Inizializzazione della rete. */
	sink = parentModule()->findSubmodule("sink");
	
	/*Registrazione modulo.*/
	Packet * temp = new Packet("REGISTER");
	temp->setSource(address);
	temp->setDestination(-2);
	temp->setByteLength(0);
	send(temp,"out");
	/*Inizializzazione timer.*/
	if(T>0)
		scheduleAt(simTime()+T, new cMessage("SRCtimer"));
}

void Chaoticsource::handleMessage(cMessage *msg){
	if(msg->isSelfMessage()){
		/*Aggiornamento della mappa caotica*/
		if(x < d){ 
			//aggiornamento della variabile di stato x
			x = x + pow(x/d,m1)*(1-d);
			//non viene inviato nessun pacchetto
			y = 0;
		}
		else{
			//aggiornamento della variabile di stato x
			x = x - pow((1-x)/(1-d),m2)*d;
			//viene inviato un pacchetto al nodo sink
			y = 1;
			Packet * temp = new Packet("NOISE");
			temp->setSource(address);
			temp->setDestination(sink);
			temp->setByteLength(wlen);
			temp->setPckType(OTHER);
			send(temp,"out");
		}
		if(!youtFile){ev << "SOURCE: Errore apertura file" << endl;}
		else{youtFile << y << endl;	}
		if(!xoutFile){ev << "SOURCE: Errore apertura file" << endl;}
		else{xoutFile << x << endl; }
		/*Aggiorno il timer*/
		scheduleAt(simTime()+T, new cMessage("SRCtimer"));
	}
	/*Eliminazione del messaggio*/
	delete msg;
}

void Chaoticsource::finish(){
	closefile();
}

void Chaoticsource::openfile(){
	youtFile.open("yseries.txt");
	if(!youtFile){
		ev << "Il file non puo` essere aperto\n";
	}
	else{
		ev << "Il file e` stato aperto con successo\n";
	}
	xoutFile.open("xseries.txt");
	if(!xoutFile){
		ev << "Il file non puo` essere aperto\n";
	}
	else{
		ev << "Il file e` stato aperto con successo\n";
	}
}

void Chaoticsource::closefile(){
	youtFile.close();
	xoutFile.close();
}