/*
Copyright(C) 2007 Giada Giorgi

This file is part of X-Simulator.

    X-Simulator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    X-Simulator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <omnetpp.h>
#include "Packet_m.h"
#include "Event_m.h"
#include "Constant.h"

class Master:public cSimpleModule{
protected:
	virtual void initialize();
	virtual void handleMessage(cMessage *msg);
	virtual void finish();
private:
	// ---------------------------------------------------------------------------
	// Dichiarazione metodi.
	// ---------------------------------------------------------------------------
	void handleSelfMessage(cMessage *msg);
	void handleSlaveMessage(cMessage *msg);
	// ---------------------------------------------------------------------------
	// Dichiarazione variabili.
	// ---------------------------------------------------------------------------
	const char *name;
	int address;
	// ---------------------------------------------------------------------------
	// Variabili di stato.
	// ---------------------------------------------------------------------------
	double Tsync;
};

Define_Module(Master);

void Master::initialize(){
	// ---------------------------------------------------------------------------
	// Inizializzazione variabili.
	// ---------------------------------------------------------------------------
	Tsync = par("Tsync");
	// ---------------------------------------------------------------------------
	// Nome e Id del nodo.
	// ---------------------------------------------------------------------------
	name = "master";
	address = id();
	// ---------------------------------------------------------------------------
	// Inizializzazione del timer.
	// ---------------------------------------------------------------------------
	scheduleAt(simTime()+Tsync, new cMessage("MStimer"));
	// ---------------------------------------------------------------------------
	// Registrazione nodo nella rete.
	// ---------------------------------------------------------------------------
	Packet * temp = new Packet("REGISTER");
	temp->setSource(address);
	temp->setDestination(-2);
	temp->setByteLength(0);
	send(temp,"out");
}

void Master::handleMessage(cMessage *msg){
	if(msg->isSelfMessage()){handleSelfMessage(msg);}
	if(msg->arrivedOn("in"))
		//Un messaggio arrivato da questa porta  sicuramente un pacchetto
		if(((Packet*)msg)->getSource()!=address & 
			(((Packet*)msg)->getDestination()==address |
			((Packet*)msg)->getDestination()==-1))
			if((((Packet *)msg)->getPckType())==PTP)
				handleSlaveMessage(msg);
	delete msg;
}

/*********************************************************************************
 METODI PRIVATI
**********************************************************************************/
void Master::handleSelfMessage(cMessage *msg){
	Packet *pck = new Packet("SYNC");
	pck->setPtpType(SYNC);
	pck->setClockType(TIME_REQ);
	pck->setByteLength(SYNC_BYTE);
	pck->setTimestamp(simTime());
	pck->setSource(address);
	pck->setData(simTime());
	pck->setDestination(-1); 
	send(pck,"out");
	scheduleAt(simTime()+Tsync, new cMessage("MStimer"));
}

void Master::handleSlaveMessage(cMessage *msg){
	
	switch (((Packet *)msg)->getPtpType()){
		case SYNC:
		case DRES:
			error("Invalid slave message");
			break;
		case DREQ:
			Packet *pck = new Packet("DRES");
			pck->setByteLength(DRES_BYTE);
			pck->setDestination(((Packet *)msg)->getSource());
			pck->setSource(address);
			pck->setPtpType(DRES);
			pck->setData(simTime());
			pck->setPtpType(DRES);
			send(pck,"out");
			break;
	}
}

void Master::finish(){
}